#include <fstream.h>
#include <windows.h>
#include <gl\gl.h>
#include <gl\glu.h>
#include <math.h>

#define N            50
#define M            50
#define MIN	         (M_PI * 2)
#define FOG_DENSITY  0.08
#define Z_RANGE		100
#define TEX_NAME	   "texture.bmp"
#define TEX_WIDTH		256
#define TEX_HEIGHT	256
#define S_FACTOR     0.2

UINT timer;
char *texture;
GLuint textures[1];

const GLfloat no_mat[] = { 0.0, 0.0, 0.0, 1.0 };
const GLfloat surf_color[] = { 1.0, 1.0, 1.0, 1.0 };
const GLfloat mat_specular[] = { 1.0, 1.0, 1.0, 1.0 };
const GLfloat mat_ambient[] = { 0.5, 0.5, 0.5, 1.0 };
const GLfloat low_shininess[] = { 2.0 };

void setDCPixelFormat(HDC hdc)
{
  static PIXELFORMATDESCRIPTOR pfd =
  {
    sizeof(PIXELFORMATDESCRIPTOR),          // sizeof this structure
    1,                                      // version number
    PFD_DRAW_TO_WINDOW | PFD_SUPPORT_OPENGL
    | PFD_DOUBLEBUFFER,                     // flags
    PFD_TYPE_RGBA,                          // RGBA pixel values
    16,                                     // 24-bit color
    0, 0, 0, 0, 0, 0,                       // don't care about these
    0, 0,                                   // no alpha buffer
    0, 0, 0, 0, 0,                          // no accumulation buffer
    16,                                     // 32-bit depth buffer
    0,                                      // no stencil buffer
    0,                                      // no auxiliary buffers
    PFD_MAIN_PLANE,                         // layer type
    0,                                      // reserved(must be 0)
    0, 0, 0                                 // no layer masks
  };
  int pixelFormat;

  pixelFormat = ChoosePixelFormat(hdc, &pfd);
  SetPixelFormat(hdc, pixelFormat, &pfd);
  DescribePixelFormat(hdc, pixelFormat, sizeof(PIXELFORMATDESCRIPTOR), &pfd);
}

void InitializeGL()
{
  GLfloat ambient[] = { 0.0, 0.0, 0.0, 1.0 };
  GLfloat diffuse[] = { 1.0, 1.0, 1.0, 1.0 };
  GLfloat position[] = { 0.0, 0.0, 5.0, 0.0 };
  GLfloat lmodel_ambient[] = { 0.4, 0.4, 0.4, 1.0 };
  GLfloat local_view[] = { 0.0 };
  GLfloat fogColor[4] = {0.0, 0.25, 0.0, 1.0};

  glFrontFace(GL_CW);
  glCullFace(GL_BACK);
  glEnable(GL_CULL_FACE);
  glEnable(GL_SMOOTH);

  glDepthFunc(GL_LEQUAL);
  glEnable(GL_DEPTH_TEST);
  glClearColor(fogColor[0], fogColor[1], fogColor[2], fogColor[3]);

  glLightfv(GL_LIGHT0, GL_AMBIENT, ambient);
  glLightfv(GL_LIGHT0, GL_DIFFUSE, diffuse);
  glLightfv(GL_LIGHT0, GL_POSITION, position);
  glLightModelfv(GL_LIGHT_MODEL_AMBIENT, lmodel_ambient);
  glLightModelfv(GL_LIGHT_MODEL_LOCAL_VIEWER, local_view);

  glEnable(GL_LIGHTING);
  glEnable(GL_LIGHT0);

//  glGenTextures(1, textures);
//  glPixelStorei(GL_UNPACK_ALIGNMENT,1);
//  glBindTexture(GL_TEXTURE_2D, textures[0]);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
  glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);
  glTexImage2D(GL_TEXTURE_2D, 0, 3, TEX_WIDTH, TEX_HEIGHT, 0, GL_RGB, GL_UNSIGNED_BYTE, texture);
  glEnable(GL_TEXTURE_2D);

  glFogf(GL_FOG_MODE, GL_EXP2);
  glFogf(GL_FOG_DENSITY, FOG_DENSITY);
  glFogf(GL_FOG_END, Z_RANGE);
  glFogfv(GL_FOG_COLOR, fogColor);
  glEnable(GL_FOG);
}

void DrawSurface(GLfloat offset)
{
  GLfloat x, y = -MIN, z, C1 = MIN * 2.0 / (GLfloat)M, C2 = MIN * 2.0 / (GLfloat)N;
  GLfloat n_x, n_y, n_z, last_x, last_y = y;
  GLfloat sin_x, sin_y, last_sin_x, last_sin_y = sin(y), t;
  GLfloat cos_x, cos_y, last_cos_x, last_cos_y = cos(y);
  GLfloat n1, n2, n3, n4, n5, n6, n7, n8;
  GLfloat x_, y_, last_x_, last_y_ = (y + offset) * S_FACTOR;

  glBegin(GL_TRIANGLES);
    for (int i = 0; i < N; i++)
    {
      x = -MIN;
      last_x = x;
      sin_y = sin(y + offset);
      cos_y = cos(y + offset);
      last_sin_x = sin(x + offset);
      last_cos_x = cos(x + offset);
      last_x_ = (x + offset) * S_FACTOR;
      y_ = (y + offset) * S_FACTOR;
      for (int j = 0; j < M; j++)
      {
        sin_x = sin(x + offset);
        cos_x = cos(x + offset);

        z = sin_x * sin_y;
        t = last_sin_x * last_sin_y;

        n1 = -last_cos_x * sin_y;
        n2 = -last_sin_x * cos_y;
        n3 = -cos_x * sin_y;
        n4 = -sin_x * cos_y;
        n5 = -last_cos_x * last_sin_y;
        n6 = -last_sin_x * last_cos_y;
        n7 = -cos_x * last_sin_y;
        n8 = -sin_x * last_cos_y;

        x_ = (x + offset) * S_FACTOR;

        glNormal3f(n1, n2, 1.0);
        glTexCoord2f(last_x_, y_);
        glVertex3f(last_x, y, last_sin_x * sin_y);
        glNormal3f(n3, n4, 1.0);
        glTexCoord2f(x_, y_);
        glVertex3f(x, y, z);
        glNormal3f(n5, n6, 1.0);
        glTexCoord2f(last_x_, last_y_);
        glVertex3f(last_x, last_y, t);

        glVertex3f(last_x, last_y, t);
        glNormal3f(n3, n4, 1.0);
        glTexCoord2f(x_, y_);
        glVertex3f(x, y, z);
        glNormal3f(n7, n8, 1.0);
        glTexCoord2f(x_, last_y_);
        glVertex3f(x, last_y, sin_x * last_sin_y);

        last_x = x;
        last_sin_x = sin_x;
        last_cos_x = cos_x;
        last_x_ = x_;
        x += C1;
      }
      last_y = y;
      last_sin_y = sin_y;
      last_cos_y = cos_y;
      last_y_ = y_;
      y += C2;
    }
  glEnd();
}

void drawScene(HDC hDC)
{
  static GLfloat angle = 0;

  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
  glTranslatef(0, 0, -12);
  glRotatef(-60, 1, 0, 0);

  glMaterialfv(GL_FRONT, GL_AMBIENT, mat_ambient);
  glMaterialfv(GL_FRONT, GL_SPECULAR, mat_specular);
  glMaterialfv(GL_FRONT, GL_EMISSION, no_mat);
  glMaterialfv(GL_FRONT, GL_SHININESS, low_shininess);
  glMaterialfv(GL_FRONT, GL_DIFFUSE, surf_color);

  DrawSurface(angle);

  SwapBuffers(hDC);
  glFlush();

  angle += 0.1;
}

LONG WINAPI wndProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
  static HDC hdc;
  static HGLRC hrc;
  PAINTSTRUCT ps;
  GLdouble aspect;
  GLsizei width, height;

  switch(msg)
  {
    case WM_CREATE:
      hdc = GetDC(hWnd);
      setDCPixelFormat(hdc);
      hrc = wglCreateContext(hdc);
      wglMakeCurrent(hdc, hrc);
      InitializeGL();
      timer = SetTimer(hWnd, 1, 50, NULL);
      return 0;

    case WM_SIZE:
      width = (GLsizei)LOWORD(lParam);
      height = (GLsizei)HIWORD(lParam);
      if (!width || !height) return 0;

      aspect = (GLdouble)width /(GLdouble)height;
      glMatrixMode(GL_PROJECTION);
      glLoadIdentity();
      gluPerspective(30.0, aspect, 1.0, 50.0);
      glViewport(0, 0, width, height);

      return 0;

    case WM_PAINT:
      BeginPaint(hWnd, &ps);
      drawScene(hdc);
      EndPaint(hWnd, &ps);
      return 0;

    case WM_DESTROY:
      wglMakeCurrent(NULL, NULL);
      wglDeleteContext(hrc);
      KillTimer(hWnd, timer);
      PostQuitMessage(0);
      return 0;

    case WM_TIMER:
      InvalidateRect(hWnd, NULL, FALSE);
      return 0;
  }

  return DefWindowProc(hWnd, msg, wParam, lParam);
}

int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR cmdLine, int cmdShow)
{
  static char appName [] = "Surface - by HackMaster";
  WNDCLASS wc;
  HWND hWnd;
  MSG msg;

  texture = new char[TEX_WIDTH * TEX_HEIGHT * 3];
  ifstream in(TEX_NAME, ios::binary);
  if (!in)
  {
    MessageBox(0, " TEXTURE.BMP  ", "", MB_OK | MB_ICONSTOP);
    return 1;
  }
  in.seekg(0x36, ios::beg);
  in.read(texture, TEX_WIDTH * TEX_HEIGHT * 3);
  in.close();

  wc.style = CS_HREDRAW | CS_VREDRAW;
  wc.lpfnWndProc =(WNDPROC)wndProc;
  wc.cbClsExtra = 0;
  wc.cbWndExtra = 0;
  wc.hInstance = hInstance;
  wc.hIcon = LoadIcon(NULL, IDI_APPLICATION);
  wc.hCursor = LoadCursor(NULL, IDC_ARROW);
  wc.hbrBackground =(HBRUSH)(COLOR_WINDOW + 1);
  wc.lpszMenuName = (LPCTSTR)0;
  wc.lpszClassName = appName;

  RegisterClass(&wc);

  hWnd = CreateWindow(appName, appName,
                      WS_OVERLAPPEDWINDOW | WS_CLIPCHILDREN | WS_CLIPSIBLINGS,
                      CW_USEDEFAULT, CW_USEDEFAULT, 320,
                      240, HWND_DESKTOP, NULL, hInstance, NULL);

  ShowWindow(hWnd, cmdShow);
  UpdateWindow(hWnd);

  while(GetMessage(&msg, NULL, 0, 0))
  {
    TranslateMessage(&msg);
    DispatchMessage(&msg);
  }

  return msg.wParam;
}

